#include "Arrow.h"
#include "SFVec3f.h"
#include "SFColor.h"
#include <new.h>

Arrow::Arrow()
{
}

Arrow::Arrow(const SFVec3f& position, 
	     const SFVec3f& direction,	 
	     const SFFloat& length,  
	     const SFFloat& impact, 
	     const SFColor& rgbColor):
    _aLine(length,
	   position,
	   direction),
    _aSphere(impact / 600000,
	     position),
    _aCone(impact / 300000,
	   impact / 30000,
	   position + direction.unit() * length,
	   direction),
    _aColor(rgbColor),
    _aCenter(position)
{
    MFFloat range;
    range.append(20 * length);

    _aLineSphereCone.add(&_aLine);
    _aLineSphereCone.add(&_aSphere);
    _aLineSphereCone.add(&_aCone);

    _aLOD.~ThreeDLOD();
    new (&_aLOD) ThreeDLOD(range, _aCenter);
    _aLOD.add(&_aLineSphereCone);
    _aLOD.add(&_aLine);

    add(&_aColor);
    add(&_aLOD);
}

Arrow::Arrow(const Arrow& original):
    _aLine(original._aLine),
    _aSphere(original._aSphere),
    _aCone(original._aCone),
    _aColor(original._aColor),
    _aLineSphereCone(original._aLineSphereCone),
    _aLOD(original._aLOD),
    _aCenter(original._aCenter)
{
}

Arrow::~Arrow()
{
}

Arrow& Arrow::operator=(const Arrow& original)
{
    _aLine = original._aLine;
    _aSphere = original._aSphere;
    _aCone = original._aCone;
    _aColor = original._aColor;
    _aLineSphereCone = original._aLineSphereCone;
    _aLOD = original._aLOD;
    _aCenter = original._aCenter;
    return(*this);
}





