#include "Line.h"
#include "SFVec3f.h"
#include <new.h>
#include <math.h>

Line::Line()
{
}

Line::Line(const SFVec3f& from,
	   const SFVec3f& to):
    _coordIndex(0),
    _aCoordinate3(from),
    _anIndexedLineSet(_coordIndex, -1, -1, -1)
{
    _aCoordinate3.add(to);
    _coordIndex.append(1);
    _coordIndex.append(-1);
    _anIndexedLineSet.~ThreeDIndexedLineSet();
    new (&_anIndexedLineSet) ThreeDIndexedLineSet(_coordIndex, -1, -1, -1);

    add(&_aCoordinate3);
    add(&_anIndexedLineSet);
}

Line::Line(const SFFloat& length,
	   const SFVec3f& position,
	   const SFVec3f& direction):
    _coordIndex(0),
    _aCoordinate3(),
    _anIndexedLineSet(_coordIndex, -1, -1, -1),
    _aTranslation(position)
{
    SFVec3f zero(0, 0, 0);
    SFVec3f to(0, length, 0);
    _aCoordinate3.add(zero);
    _aCoordinate3.add(to);
    _coordIndex.append(1);
    _coordIndex.append(-1);
    _anIndexedLineSet.~ThreeDIndexedLineSet();
    new (&_anIndexedLineSet) ThreeDIndexedLineSet(_coordIndex, -1, -1, -1);

    SFVec3f yAxis(0, 1, 0); 
    SFVec3f anAxis(direction.z(), 0, -direction.x());
    SFFloat anAngle = acos(direction.y() / direction.mag());
    SFRotation aRot(anAxis, anAngle);
    _aRotation.~ThreeDRotation();
    new (&_aRotation) ThreeDRotation(aRot);

    add(&_aTranslation);
    add(&_aRotation);
    add(&_aCoordinate3);
    add(&_anIndexedLineSet);
}

Line::Line(const Line& original):
    _coordIndex(original._coordIndex),
    _aCoordinate3(original._aCoordinate3),
    _anIndexedLineSet(original._anIndexedLineSet),
    _aRotation(original._aRotation),
    _aTranslation(original._aTranslation)
{
}

Line::~Line()
{
}

Line& Line::operator=(const Line& original)
{
    _coordIndex = original._coordIndex;
    _aCoordinate3 = original._aCoordinate3;
    _anIndexedLineSet = original._anIndexedLineSet;
    _aRotation = original._aRotation;
    _aTranslation = original._aTranslation;
    return(*this);
}







